"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.start = start;
var colorette = _interopRequireWildcard(require("colorette"));
var _package = _interopRequireDefault(require("../../../package.json"));
var _env = require("../../env.js");
var _index = require("../../logging/index.js");
var _index2 = require("../common/index.js");
var _consts = require("../consts.js");
var _Compiler = require("./Compiler.js");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
// @ts-expect-error type-only import

/**
 * Start command for React Native Community CLI.
 * It runs `@callstack/repack-dev-server` to provide Development Server functionality to React Native apps
 * in development mode.
 *
 * @param _ Original, non-parsed arguments that were provided when running this command.
 * @param config React Native Community CLI configuration object.
 * @param args Parsed command line arguments.
 *
 * @internal
 * @category CLI command
 */
async function start(_, config, args) {
  const webpackConfigPath = (0, _index2.getWebpackConfigFilePath)(config.root, args.config ?? args.webpackConfig);
  const {
    reversePort,
    ...restArgs
  } = args;
  const serverProtocol = args.https ? 'https' : 'http';
  const serverHost = args.host || _consts.DEFAULT_HOSTNAME;
  const serverPort = args.port ?? _consts.DEFAULT_PORT;
  const serverURL = `${serverProtocol}://${serverHost}:${serverPort}`;
  const showHttpRequests = args.verbose || args.logRequests;
  const cliOptions = {
    config: {
      root: config.root,
      platforms: Object.keys(config.platforms),
      bundlerConfigPath: webpackConfigPath,
      reactNativePath: config.reactNativePath
    },
    command: 'start',
    arguments: {
      start: {
        ...restArgs,
        host: serverHost,
        port: serverPort
      }
    }
  };
  if (args.platform && !cliOptions.config.platforms.includes(args.platform)) {
    throw new Error('Unrecognized platform: ' + args.platform);
  }
  if (args.verbose) {
    process.env[_env.VERBOSE_ENV_KEY] = '1';
  }
  const reporter = (0, _index.composeReporters)([new _index.ConsoleReporter({
    asJson: args.json,
    isVerbose: args.verbose
  }), args.logFile ? new _index.FileReporter({
    filename: args.logFile
  }) : undefined].filter(Boolean));
  const version = _package.default.version;
  process.stdout.write(colorette.bold(colorette.cyan('📦 Re.Pack ' + version + '\n\n')));
  const compiler = new _Compiler.Compiler(cliOptions, reporter);
  const {
    createServer
  } = await import('@callstack/repack-dev-server');
  const {
    start,
    stop
  } = await createServer({
    options: {
      rootDir: cliOptions.config.root,
      host: serverHost,
      port: serverPort,
      https: args.https ? {
        cert: args.cert,
        key: args.key
      } : undefined,
      logRequests: showHttpRequests
    },
    delegate: ctx => {
      if (args.interactive) {
        (0, _index2.setupInteractions)({
          onReload() {
            ctx.broadcastToMessageClients({
              method: 'reload'
            });
          },
          onOpenDevMenu() {
            ctx.broadcastToMessageClients({
              method: 'devMenu'
            });
          },
          onOpenDevTools() {
            void fetch(`${serverURL}/open-debugger`, {
              method: 'POST'
            });
          },
          onAdbReverse() {
            void (0, _index2.runAdbReverse)({
              port: serverPort,
              logger: ctx.log,
              verbose: true
            });
          }
        }, {
          logger: ctx.log
        });
      }
      if (reversePort) {
        void (0, _index2.runAdbReverse)({
          logger: ctx.log,
          port: serverPort,
          wait: true
        });
      }
      const lastStats = {};
      compiler.on('watchRun', ({
        platform
      }) => {
        ctx.notifyBuildStart(platform);
        if (platform === 'android') {
          void (0, _index2.runAdbReverse)({
            port: serverPort,
            logger: ctx.log
          });
        }
      });
      compiler.on('invalid', ({
        platform
      }) => {
        ctx.notifyBuildStart(platform);
        ctx.broadcastToHmrClients({
          action: 'building'
        }, platform);
      });
      compiler.on('done', ({
        platform,
        stats
      }) => {
        ctx.notifyBuildEnd(platform);
        lastStats[platform] = stats;
        ctx.broadcastToHmrClients({
          action: 'built',
          body: createHmrBody(stats)
        }, platform);
      });
      return {
        compiler: {
          getAsset: (filename, platform, sendProgress) => {
            const parsedUrl = (0, _index2.parseFileUrl)(filename, 'file:///');
            return compiler.getSource(parsedUrl.filename, platform, sendProgress);
          },
          getMimeType: filename => (0, _index2.getMimeType)(filename),
          inferPlatform: uri => {
            const {
              platform
            } = (0, _index2.parseFileUrl)(uri, 'file:///');
            return platform;
          }
        },
        symbolicator: {
          getSource: fileUrl => {
            const {
              filename,
              platform
            } = (0, _index2.parseFileUrl)(fileUrl);
            return compiler.getSource(filename, platform);
          },
          getSourceMap: fileUrl => {
            const {
              filename,
              platform
            } = (0, _index2.parseFileUrl)(fileUrl);
            return compiler.getSourceMap(filename, platform);
          },
          shouldIncludeFrame: frame => {
            // If the frame points to internal bootstrap/module system logic, skip the code frame.
            return !/webpack[/\\]runtime[/\\].+\s/.test(frame.file);
          }
        },
        hmr: {
          getUriPath: () => '/__hmr',
          onClientConnected: (platform, clientId) => {
            ctx.broadcastToHmrClients({
              action: 'sync',
              body: createHmrBody(lastStats[platform])
            }, platform, [clientId]);
          }
        },
        messages: {
          getHello: () => 'React Native packager is running',
          getStatus: () => 'packager-status:running'
        },
        logger: {
          onMessage: log => {
            const logEntry = (0, _index.makeLogEntryFromFastifyLog)(log);
            logEntry.issuer = 'DevServer';
            reporter.process(logEntry);
          }
        },
        api: {
          getPlatforms: () => Promise.resolve(Object.keys(compiler.workers)),
          getAssets: platform => Promise.resolve(Object.entries(compiler.assetsCache[platform] ?? {}).map(([name, asset]) => ({
            name,
            size: asset.size
          }))),
          getCompilationStats: platform => Promise.resolve(compiler.statsCache[platform] ?? null)
        }
      };
    }
  });
  await start();
  return {
    stop: async () => {
      reporter.stop();
      await stop();
    }
  };
}
function createHmrBody(stats) {
  if (!stats) {
    return null;
  }
  return {
    name: stats.name ?? '',
    time: stats.time ?? 0,
    hash: stats.hash ?? '',
    warnings: stats.warnings || [],
    errors: stats.errors || []
  };
}
//# sourceMappingURL=start.js.map